@@ -7,7 +7,7 @@
    version 2 of the License, or (at your option) any later version.
 */
 
-#include "pcx.h"
+#include "pcx_p.h"
 
 #include <QColor>
 #include <QDataStream>
@@ -667,10 +667,10 @@ QImageIOPlugin::Capabilities PCXPlugin::capabilities(QIODevice *device, const QB
         return Capabilities(CanRead | CanWrite);
     }
     if (!format.isEmpty()) {
-        return 0;
+        return nullptr;
     }
     if (!device->isOpen()) {
-        return 0;
+        return nullptr;
     }
 
     Capabilities cap;
